@@ -4,6 +4,7 @@
  */
 #include "proc/version.h"
 #include "proc/readproc.h"
+#include "proc/devname.h"
 #include "proc/ps.h"
 #include <stdio.h>
 #include <stdlib.h>
@@ -113,7 +114,7 @@ int file2str(char *directory, char *what, char *ret, int cap) {
 
     sprintf(filename, "%s/%s", directory, what);
     if ( (fd       = open(filename, O_RDONLY, 0)) == -1 ) return -1;
-    if ( (num_read = read(fd, ret, cap - 1))      == -1 ) return -1;
+    if ( (num_read = read(fd, ret, cap - 1))      <= 0 ) return -1;
     ret[num_read] = 0;
     close(fd);
     return num_read;
@@ -123,6 +124,7 @@ char** file2strvec(char* directory, char* what) {
     char buf[2048];	/* read buf bytes at a time */
     char *p, *rbuf = 0, *endbuf, **q, **ret;
     int fd, tot = 0, n, c, end_of_file = 0;
+    int align;
 
     sprintf(buf, "%s/%s", directory, what);
     if ( (fd = open(buf, O_RDONLY, 0) ) == -1 ) return NULL;
@@ -131,6 +133,8 @@ char** file2strvec(char* directory, char* what) {
     while ((n = read(fd, buf, sizeof buf - 1)) > 0) {
 	if (n < sizeof buf - 1)
 	    end_of_file = 1;
+	if (n == 0 && rbuf == 0)
+	    return NULL;	/* process died between our open and read */
 	if (n < 0) {
 	    if (rbuf)
 		free(rbuf);
@@ -150,14 +154,15 @@ char** file2strvec(char* directory, char* what) {
 	return NULL;		/* read error */
     }
     endbuf = rbuf + tot;			/* count space for pointers */
+    align = (sizeof(char*)-1) - ((tot + sizeof(char*)-1) & (sizeof(char*)-1));
     for (c = 0, p = rbuf; p < endbuf; p++)
     	if (!*p)
 	    c += sizeof(char*);
     c += sizeof(char*);				/* one extra for NULL term */
 
-    rbuf = realloc(rbuf, tot + c);		/* make room for ptrs AT END */
+    rbuf = realloc(rbuf, tot + c + align);	/* make room for ptrs AT END */
     endbuf = rbuf + tot;			/* addr just past data buf */
-    q = ret = (char**) endbuf;			/* ==> free(*ret) to dealloc */
+    q = ret = (char**) (endbuf+align);		/* ==> free(*ret) to dealloc */
     *q++ = p = rbuf;				/* point ptrs to the strings */
     endbuf--;					/* do not traverse final NUL */
     while (++p < endbuf) 
@@ -243,7 +248,7 @@ next_proc:				/* get next PID for consideration */
     if (!matched && Do(TTY) && !XinL(dev_t, p->tty, PT->ttys))
 	goto next_proc;			/* not one of the requested ttys */
 
-    if (!matched && !Do(ANYTTY) && p->tty == -1)
+    if (!matched && Do(ANYTTY) && p->tty == -1)
 	goto next_proc;			/* no controlling terminal */
 
     if (!matched && Do(STAT) && !strchr(PT->stats,p->state))
