@@ -11,7 +11,7 @@ typedef struct proc_s {
     	user[10],	/* user name corresponding to owner of process */
     	cmd[40],	/* basename of executable file in call to exec(2) */
     	state,		/* single-char code for process state (S=sleeping) */
-    	ttyc[4],	/* string representation of controlling tty device */
+    	ttyc[5],	/* string representation of controlling tty device */
 	**environ,	/* environment string vector (/proc/#/environ) */
 	**cmdline;	/* command line string vector (/proc/#/cmdline) */
     int
@@ -57,7 +57,8 @@ typedef struct proc_s {
 	start_stack,	/* address of the bottom of stack for the process */
 	kstk_esp,	/* kernel stack pointer */
 	kstk_eip,	/* kernel stack pointer */
-	wchan;		/* address of kernel wait channel proc is sleeping in */
+	wchan,		/* address of kernel wait channel proc is sleeping in */
+        pcpu;           /* %CPU usage (is not filled in by readproc!!!) */
     struct proc_s *l,	/* ptrs for building arbitrary linked structs */
                   *r;	/* (i.e. singly/doubly-linked lists and trees */
 } proc_t;
