@@ -16,12 +16,13 @@
  *   and   Copyright (C) 1995, 1996 Charles Blake
  * See file COPYING for copyright details.
  */
+#include <proc/version.h>
+#include <proc/readproc.h>
 #include <proc/ps.h>
 #include <proc/psdata.h>
-#include <proc/readproc.h>
+#include <proc/devname.h>
 #include <proc/tree.h>
 #include <proc/sysinfo.h>
-#include <proc/version.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -54,7 +55,7 @@ struct {
     const char*	header;
 } mode[] = {
     { show_short,  0 , "Up", "  PID TTY STAT  TIME COMMAND" },
-    { show_long,  'l', "Pp", " F    UID   PID  PPID PRI NI  SIZE   RSS WCHAN      STAT TTY   TIME COMMAND" },
+    { show_long,  'l', "Pp", " FLAGS   UID   PID  PPID PRI  NI   SIZE   RSS WCHAN       STA TTY TIME COMMAND" },
     { show_user,  'u', "up", "USER       PID %CPU %MEM  SIZE   RSS TTY STAT START   TIME COMMAND" },
     { show_jobs,  'j', "gPp"," PPID   PID  PGID   SID TTY TPGID  STAT   UID   TIME COMMAND" },
     { show_sig,   's', "p",  "  UID   PID SIGNAL   BLOCKED  IGNORED  CATCHED  STAT TTY   TIME COMMAND" },
@@ -73,7 +74,7 @@ extern int (*sort_function[])(/* void* a, void* b */);
 
 int    parse_sort_opt (const char*);
 int    parse_long_sort(const char*);
-char * status2        (proc_t*);
+char * status         (proc_t*);
 char * prtime         (char *s, unsigned long t, unsigned long rel);
 void   usage          (char* context);
 void   set_cmdspc     (int w_opts);
@@ -100,7 +101,7 @@ pid_t * CL_pids;          /* zero-terminated list, dynamically allocated */
 /* process display modification command line options */
 
 int CL_show_env,
-    CL_num_outp,          /* numeric fields for user or wchan */
+    CL_num_outp,          /* numeric fields for user, wchan, tty */
     CL_sort     = 1,
     CL_forest,
     CL_Sum,
@@ -159,14 +160,16 @@ int main(int argc, char **argv) {
 	      case 'v': CL_fmt = PS_V;				break;
 	      case 'm': CL_fmt = PS_M;				break;
 	      case 'X': CL_fmt = PS_X;   /* regs */		break;
-	      case 'f': CL_forest = 1;
-		CL_kern_comm = 0;			break;
+
 	      case 'r': CL_run_only = 1; /* list filters */	break;
 	      case 'a': CL_all = 1;				break;
 	      case 'x': CL_no_ctty = 1;				break;
 	      case 't': CL_ctty = p + 1;
 		next_arg = 1;				break;
+
 	      case 'e': CL_show_env = 1; /* output modifiers */	break;
+	      case 'f': CL_forest = 1;
+		CL_kern_comm = 0;			break;
 	      case 'c': CL_kern_comm = 1;			break;
 	      case 'w': ++width;				break;
 	      case 'h': do_header = 0;				break;
@@ -228,16 +231,16 @@ int main(int argc, char **argv) {
     /* NOTE:  all but option parsing has really been done to enable
      * multiple uid/tty/state filtering as well as multiple pid filtering
      */
-    pflags = PROC_FILLTTY | PROC_ANYTTY;	/* defaults */
+    pflags = PROC_ANYTTY;	/* defaults */
 
     if (!CL_kern_comm)	pflags |= PROC_FILLCMD;  	 /* verbosity flags */
     if (CL_fmt == PS_M) pflags |= PROC_FILLMEM;
     if (CL_show_env)	pflags |= PROC_FILLENV;
-    if (!CL_num_outp)	pflags |= PROC_FILLUSR;
+    if (!CL_num_outp)	pflags |= PROC_FILLUSR | PROC_FILLTTY;
 
-    if (CL_no_ctty)	pflags |= PROC_ANYTTY;		/* filter flags */
+    if (CL_no_ctty)	pflags &= ~PROC_ANYTTY;		/* filter flags */
     if (CL_run_only)  { pflags |= PROC_STAT; args = "RD"; }
-    if (!CL_all)      { pflags |= PROC_UID;  args = uid; uid[0] = getuid(); pflags &= ~PROC_STAT; }
+    else if (!CL_all)      { pflags |= PROC_UID;  args = uid; uid[0] = getuid(); pflags &= ~PROC_STAT; }
     if (CL_pids)      { pflags |= PROC_PID;  args = CL_pids; pflags &= ~PROC_UID; pflags &= ~PROC_STAT; }
     if (CL_ctty) {
 	if ((tty[0] = tty_to_dev(CL_ctty)) == (dev_t)-1) {
@@ -304,6 +307,8 @@ void show_procs(unsigned maxcmd, int do_header, int pflags, void* args, int N) {
 	n++;					/* Now either: */
 	if (CL_forest) {			/*    add process to tree */
 	    static char s[256];
+	    if (CL_num_outp)
+		snprintf(next->ttyc, sizeof next->ttyc, "%04.4x", next->tty);
 	    (mode[CL_fmt].format)(s, next);
 	    if (CL_fmt != PS_V && CL_fmt != PS_M)
 		show_time(s+strlen(s), next);
@@ -362,6 +367,8 @@ void show_cmd_env(char* tskcmd, char** cmd, char** env, unsigned maxch) {
  */
 void show_a_proc(proc_t* p, unsigned maxch) {
     static char s[2048];
+    if (CL_num_outp)
+	snprintf(p->ttyc, sizeof p->ttyc, "%04.4x", p->tty);
     (mode[CL_fmt].format)(s, p);
     if (CL_fmt != PS_V && CL_fmt != PS_M)
 	show_time(s+strlen(s), p);
@@ -372,24 +379,24 @@ void show_a_proc(proc_t* p, unsigned maxch) {
 /* The format functions for the various formatting modes follow */
 
 void show_short(char *s, proc_t *p) {
-    sprintf(s, "%5d %3s %s", p->pid, p->ttyc, status2(p));
+    sprintf(s, "%5d %3s %s", p->pid, p->ttyc, status(p));
 }
 
 void show_long(char *s, proc_t *p) {
     char wchanb[10];
     
     if (GL_wchan_nout)
-	sprintf(wchanb, "%-9x", p->wchan);
+	sprintf(wchanb, " %-9x ", p->wchan);
     else
-	sprintf(wchanb, "%-9.9s", wchan(p->wchan));
-    sprintf(s, "%3x %5d %5d %5d %3d %2d %5d %5d %-10.10s %s %3s ",
+	sprintf(wchanb, "%-11.11s", wchan(p->wchan));
+    sprintf(s, "%6x %5d %5d %5d %3d %3d %6d %5d %-11.11s %s%3s",
 	    p->flags, p->uid, p->pid, p->ppid, p->priority, p->nice,
-	    p->vsize >> 10, p->rss * 4, wchanb, status2(p), p->ttyc);
+	    p->vsize >> 10, p->rss * 4, wchanb, status(p), p->ttyc);
 }
 
 void show_jobs(char *s, proc_t *p) {
     sprintf(s, "%5d %5d %5d %5d %3s %5d  %s %5d ",
-	    p->ppid, p->pid, p->pgrp, p->session, p->ttyc, p->tpgid, status2(p),
+	    p->ppid, p->pid, p->pgrp, p->session, p->ttyc, p->tpgid, status(p),
 	    p->uid);
 }
 
@@ -413,26 +420,26 @@ void show_user(char *s, proc_t *p) {
     pmem = p->rss * 1000 / (GL_main_mem >> 12);
     sprintf(s, "%5d %2u.%u %2d.%d %5d %5d %2s %s%.6s ",
 	    p->pid,  pcpu / 10, pcpu % 10,  pmem / 10, pmem % 10,
-	    p->vsize >> 10, p->rss << 2, p->ttyc, status2(p),
+	    p->vsize >> 10, p->rss << 2, p->ttyc, status(p),
 	    ctime(&start) + (GL_time_now - start > 3600*24 ? 4 : 10));
 }
 
 void show_sig(char *s, proc_t *p) {
     sprintf(s, "%5d %5d %08x %08x %08x %08x %s %3s ",
 	    p->uid, p->pid, p->signal, p->blocked, p->sigignore, p->sigcatch,
-	    status2(p), p->ttyc);
+	    status(p), p->ttyc);
 }
 
 void show_vm(char *s, proc_t *p) {
     int pmem;
 
-    s += sprintf(s,"%5d %3s %s", p->pid, p->ttyc, status2(p));
+    s += sprintf(s,"%5d %3s %s", p->pid, p->ttyc, status(p));
     show_time(s, p);
     s += strlen(s);
     s += sprintf(s, " %6d %4d %4d %4d ",
 		 p->maj_flt + (CL_Sum ? p->cmaj_flt : 0),
-		 (p->end_code - p->start_code) >> 10,
-		 (p->vsize - p->end_code + p->start_code) >> 10,
+		 p->vsize ? (p->end_code - p->start_code) >> 10 : 0,
+		 p->vsize ? (p->vsize - p->end_code + p->start_code) >> 10 : 0,
 		 p->rss << 2);
     if(p->rss_rlim == RLIM_INFINITY)
 	s += sprintf(s, "   xx ");
@@ -464,9 +471,9 @@ void show_regs(char *s, proc_t *p) {
     sprintf(s, "%2d %5d %8x %8x %8x %s %s %s %3s ",
 	    p->start_code >> 26, p->pid, p->start_stack,
 	    p->kstk_esp, p->kstk_eip,
-	    prtime(time1, p->timeout, GL_current_time * 100),
+	    prtime(time1, p->timeout, GL_current_time*HZ),
 	    prtime(time2, p->it_real_value, 0),
-	    status2(p), p->ttyc);
+	    status(p), p->ttyc);
 }
 
 char *prtime(char *s, unsigned long t, unsigned long rel) {
