@@ -6,7 +6,7 @@
  * Slighty modified, and corrected, Francois Pinard, 91-04.
  */
 
-#include <ncurses.h>
+#include <curses.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
